"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generator_helper_1 = require("@prisma/generator-helper");
const sdk_1 = require("@prisma/sdk");
const node_path_1 = __importDefault(require("node:path"));
const promises_1 = require("node:fs/promises");
const lodash_1 = __importDefault(require("lodash"));
const node_fs_1 = require("node:fs");
const generator_1 = __importDefault(require("./generators/enums/generator"));
const write_file_safely_1 = __importDefault(require("./utils/write-file-safely"));
const php_cs_fixer_1 = require("./utils/php-cs-fixer");
const package_json_1 = require("../package.json");
const multiple_data_sources_error_1 = __importDefault(require("./errors/multiple-data-sources-error"));
const generator_2 = __importDefault(require("./generators/prisma-models/generator"));
const generator_3 = __importDefault(require("./generators/models/generator"));
const get_model_classname_1 = __importDefault(require("./helpers/get-model-classname"));
const delete_all_files_in_directory_1 = __importDefault(require("./helpers/delete-all-files-in-directory"));
const get_model_path_1 = __importDefault(require("./helpers/get-model-path"));
const GENERATOR_NAME = '@datomatic/prisma-laravel-generator';
(0, generator_helper_1.generatorHandler)({
    onManifest() {
        sdk_1.logger.info(`${GENERATOR_NAME}:Registered`);
        return {
            version: package_json_1.version,
            defaultOutput: '../generated',
            prettyName: GENERATOR_NAME,
        };
    },
    onGenerate: async (options) => {
        var _a, _b;
        const outputPath = (_b = (_a = options.generator.output) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : '../';
        const config = {
            prismaModelsPath: './app/Models/Prisma',
            prismaModelsPrefix: 'Prisma',
            modelsPath: './app/Models',
            prismaEnumsPath: './app/Enums/Prisma',
            baseModel: 'Illuminate\\Database\\Eloquent\\Model',
            basePivotModel: 'Illuminate\\Database\\Eloquent\\Relations\\Pivot',
            explicitTableNamesOnRelations: true,
            phpCsFixerBinPath: undefined,
            phpCsFixerConfigPath: undefined,
            ...options.generator.config,
        };
        if (options.datasources.length > 1) {
            throw new multiple_data_sources_error_1.default();
        }
        const buffer = await (0, promises_1.readFile)(options.schemaPath);
        const rawSchema = buffer.toString();
        await Promise.all([
            (0, delete_all_files_in_directory_1.default)(node_path_1.default.join(outputPath, config.prismaEnumsPath)),
            (0, delete_all_files_in_directory_1.default)(node_path_1.default.join(outputPath, config.prismaModelsPath)),
        ]);
        await Promise.all([
            ...lodash_1.default.map(options.dmmf.datamodel.enums, async (enumInfo) => {
                const generatedEnum = (0, generator_1.default)(enumInfo);
                const customPath = (0, get_model_path_1.default)(enumInfo);
                const writeLocation = node_path_1.default.join(outputPath, customPath !== null && customPath !== void 0 ? customPath : config.prismaEnumsPath, `${enumInfo.name}.php`);
                await (0, write_file_safely_1.default)(writeLocation, generatedEnum);
                if (config.phpCsFixerBinPath && config.phpCsFixerConfigPath) {
                    await (0, php_cs_fixer_1.formatFile)(writeLocation, config.phpCsFixerBinPath, config.phpCsFixerConfigPath);
                }
            }),
            ...lodash_1.default.map(options.dmmf.datamodel.models, async (model) => {
                const generatedPrismaModel = (0, generator_2.default)(model, options.dmmf.datamodel.models, options.dmmf.datamodel.enums, rawSchema, options.datasources[0].provider, config.baseModel, config.basePivotModel, config.prismaModelsPrefix, config.explicitTableNamesOnRelations);
                const writeLocation = node_path_1.default.join(outputPath, config.prismaModelsPath, `${(0, get_model_classname_1.default)(model, config.prismaModelsPrefix)}.php`);
                await (0, write_file_safely_1.default)(writeLocation, generatedPrismaModel);
                if (config.phpCsFixerBinPath && config.phpCsFixerConfigPath) {
                    await (0, php_cs_fixer_1.formatFile)(writeLocation, config.phpCsFixerBinPath, config.phpCsFixerConfigPath);
                }
            }),
            ...lodash_1.default.map(options.dmmf.datamodel.models, async (model) => {
                const generatedModel = (0, generator_3.default)(model, config.prismaModelsPrefix);
                const customPath = (0, get_model_path_1.default)(model);
                const writeLocation = node_path_1.default.join(outputPath, customPath !== null && customPath !== void 0 ? customPath : config.modelsPath, `${(0, get_model_classname_1.default)(model)}.php`);
                if (!(0, node_fs_1.existsSync)(writeLocation)) {
                    await (0, write_file_safely_1.default)(writeLocation, generatedModel);
                    if (config.phpCsFixerBinPath && config.phpCsFixerConfigPath) {
                        await (0, php_cs_fixer_1.formatFile)(writeLocation, config.phpCsFixerBinPath, config.phpCsFixerConfigPath);
                    }
                }
            }),
        ].flat());
    },
});
//# sourceMappingURL=generator.js.map