"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const prettier_1 = require("../../utils/prettier");
const get_enum_fqcn_1 = __importDefault(require("../../helpers/get-enum-fqcn"));
const get_model_traits_1 = __importDefault(require("../../helpers/get-model-traits"));
const get_class_from_fqcn_1 = __importDefault(require("../../utils/get-class-from-fqcn"));
const generateEnum = (enumInfo) => {
    const { name: enumName, values } = enumInfo;
    const imports = new Set();
    const enumValues = values
        .map(({ name: caseName, dbName }) => {
        if (dbName) {
            return `case ${caseName} = '${dbName}';`;
        }
        return `case ${caseName} = '${caseName}';`;
    })
        .join('\n');
    const { namespace } = (0, get_enum_fqcn_1.default)(enumInfo);
    const traits = new Set();
    for (const t of (0, get_model_traits_1.default)(enumInfo)) {
        imports.add(t);
        traits.add((0, get_class_from_fqcn_1.default)(t));
    }
    return (0, prettier_1.prettify)(`<?php
    namespace ${namespace};

    ${lodash_1.default.chain([...imports])
        .map(importFcqn => `use ${importFcqn};`)
        .join('\n')
        .value()}

    enum ${enumName}:string {
      ${lodash_1.default.chain([...traits])
        .map(trait => `use ${trait};`)
        .join('\n')
        .value()}

      ${enumValues}
    }
  `);
};
exports.default = generateEnum;
//# sourceMappingURL=generator.js.map