"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const get_database_field_attribute_1 = __importDefault(require("./raw-schema/get-database-field-attribute"));
const is_field_uuid_1 = __importDefault(require("../helpers/is-field-uuid"));
const is_field_enum_1 = __importDefault(require("../helpers/is-field-enum"));
const get_table_name_from_model_1 = __importDefault(require("../helpers/get-table-name-from-model"));
const get_enum_fqcn_1 = __importDefault(require("../helpers/get-enum-fqcn"));
const is_field_read_only_1 = __importDefault(require("../helpers/is-field-read-only"));
const getCastAndRulesFromField = (field, model, enums, guardedFields, fillableFields, massAssignable, rawSchema, provider) => {
    const { name: fieldName, isUnique } = field;
    const { name: modelName, uniqueFields } = model;
    const cast = {
        value: undefined,
        isMethodCall: false,
    };
    const rules = [];
    const imports = new Set();
    const isReadOnly = (0, is_field_read_only_1.default)(field, guardedFields, fillableFields, massAssignable);
    if (isUnique) {
        imports.add('Illuminate\\Validation\\Rule');
        rules.push({
            value: `Rule::unique('${(0, get_table_name_from_model_1.default)(model)}', '${fieldName}')->ignore($this->getKey(), $this->getKeyName())`,
            isMethodCall: true,
        });
    }
    if (!lodash_1.default.isEmpty(uniqueFields)) {
        const uniqueFieldsToBeAdded = lodash_1.default.filter(uniqueFields, fields => !lodash_1.default.isEmpty(fields) && fields[0] === fieldName);
        if (!lodash_1.default.isEmpty(uniqueFieldsToBeAdded)) {
            imports.add('Illuminate\\Validation\\Rule');
            for (const ufs of uniqueFieldsToBeAdded) {
                rules.push({
                    value: `Rule::unique('${(0, get_table_name_from_model_1.default)(model)}', '${fieldName}')->ignore($this->getKey(), $this->getKeyName())->where(fn ($query) => $query->${lodash_1.default.chain(ufs)
                        .slice(1)
                        .map(f => `where('${f}', $this->${f})`)
                        .join('->')
                        .value()})`,
                    isMethodCall: true,
                });
            }
        }
    }
    const { type, isList, isRequired } = field;
    if (isList) {
        cast.value = 'array';
        rules.push({
            value: 'array',
            isMethodCall: false,
        }, {
            value: 'required',
            isMethodCall: false,
        });
    }
    else {
        if (isRequired) {
            rules.push({
                value: 'required',
                isMethodCall: false,
            });
        }
        else {
            rules.push({
                value: 'nullable',
                isMethodCall: false,
            });
        }
        switch (type) {
            case 'String':
                cast.value = 'string';
                rules.push({
                    value: 'string',
                    isMethodCall: false,
                });
                break;
            case 'Boolean':
                cast.value = 'boolean';
                rules.push({
                    value: 'boolean',
                    isMethodCall: false,
                });
                break;
            case 'Int':
            case 'BigInt':
            case 'Bytes':
                cast.value = 'integer';
                rules.push({
                    value: 'numeric',
                    isMethodCall: false,
                }, {
                    value: 'integer',
                    isMethodCall: false,
                });
                break;
            case 'Float':
                cast.value = 'double';
                rules.push({
                    value: 'numeric',
                    isMethodCall: false,
                });
                break;
            case 'Decimal':
                cast.value = 'string';
                rules.push({
                    value: 'numeric',
                    isMethodCall: false,
                });
                break;
            case 'DateTime':
                cast.value = isReadOnly ? 'immutable_datetime' : 'datetime';
                rules.push({
                    value: 'date',
                    isMethodCall: false,
                });
                break;
            case 'Json':
                cast.value = 'array';
                rules.push({
                    value: 'json',
                    isMethodCall: false,
                });
                break;
            default: {
                const enumInfo = (0, is_field_enum_1.default)(field, enums);
                if (enumInfo) {
                    const { fqcn } = (0, get_enum_fqcn_1.default)(enumInfo);
                    imports.add(fqcn);
                    imports.add('Illuminate\\Validation\\Rules\\Enum');
                    cast.isMethodCall = true;
                    cast.value = `${enumInfo.name}::class`;
                    rules.push({
                        value: `new Enum(${enumInfo.name}::class)`,
                        isMethodCall: true,
                    });
                }
                break;
            }
        }
    }
    if ((0, is_field_uuid_1.default)(field)) {
        rules.push({
            value: `uuid`,
            isMethodCall: false,
        });
    }
    const databaseMapping = (0, get_database_field_attribute_1.default)(fieldName, modelName, rawSchema);
    if (databaseMapping) {
        switch (provider) {
            case 'mysql':
                if (cast.value === 'integer' &&
                    [
                        'UnsignedInt',
                        'UnsignedSmallInt',
                        'UnsignedTinyInt',
                        'UnsignedBigInt',
                    ].includes(databaseMapping.replace(/\(.*\)/, ''))) {
                    rules.push({
                        value: `min:0`,
                        isMethodCall: false,
                    });
                }
                break;
            default:
                break;
        }
    }
    return { cast, rules, imports };
};
exports.default = getCastAndRulesFromField;
//# sourceMappingURL=get-cast-and-rules-from-field.js.map