"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const is_field_enum_1 = __importDefault(require("../helpers/is-field-enum"));
const is_field_read_only_1 = __importDefault(require("../helpers/is-field-read-only"));
const get_enum_fqcn_1 = __importDefault(require("../helpers/get-enum-fqcn"));
const getPhpDocPropertiesFromField = (field, enums, guardedFields, fillableFields, massAssignable) => {
    const imports = new Set();
    let readOnly = false;
    if ((0, is_field_read_only_1.default)(field, guardedFields, fillableFields, massAssignable)) {
        readOnly = true;
    }
    const { type, isList, isRequired } = field;
    let phpType;
    switch (type) {
        case 'String':
        case 'Decimal':
            phpType = 'string';
            break;
        case 'Boolean':
            phpType = 'bool';
            break;
        case 'Int':
        case 'BigInt':
        case 'Bytes':
            phpType = 'int';
            break;
        case 'Float':
            phpType = 'double';
            break;
        case 'DateTime':
            if (readOnly) {
                imports.add('Carbon\\CarbonImmutable');
                phpType = 'CarbonImmutable';
            }
            else {
                imports.add('Illuminate\\Support\\Carbon');
                phpType = 'Carbon';
            }
            break;
        case 'Json':
            phpType = 'array';
            break;
        default: {
            const enumInfo = (0, is_field_enum_1.default)(field, enums);
            if (enumInfo) {
                const { fqcn } = (0, get_enum_fqcn_1.default)(enumInfo);
                imports.add(fqcn);
                phpType = enumInfo.name;
            }
            break;
        }
    }
    return {
        phpType: phpType
            ? `${isRequired ? '' : '?'}${phpType}${isList ? '[]' : ''}`
            : undefined,
        readOnly,
        imports,
    };
};
exports.default = getPhpDocPropertiesFromField;
//# sourceMappingURL=get-phpdoc-properties-from-field.js.map