"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBelongsToManyRelation = exports.getHasManyRelation = exports.getBelongsToRelation = exports.getHasOneRelation = void 0;
const lodash_1 = __importDefault(require("lodash"));
const is_field_relation_1 = __importDefault(require("../helpers/is-field-relation"));
const snake_1 = __importDefault(require("./strings/snake"));
const composite_key_on_relation_error_1 = __importDefault(require("../errors/composite-key-on-relation-error"));
const is_model_pivot_1 = __importDefault(require("../helpers/is-model-pivot"));
const get_table_name_from_model_1 = __importDefault(require("../helpers/get-table-name-from-model"));
const get_model_by_name_1 = __importDefault(require("../helpers/get-model-by-name"));
const get_related_field_of_related_model_1 = __importDefault(require("../helpers/get-related-field-of-related-model"));
const get_model_fqcn_1 = __importDefault(require("../helpers/get-model-fqcn"));
const get_model_classname_1 = __importDefault(require("../helpers/get-model-classname"));
const emptyOrUndefined = (value) => !value || value.length === 0;
const definedAndNotEmpty = (value) => !!value && value.length > 0;
const getHasOneRelation = (model, relationField, relatedModel, relatedField, explicitTableNames = true) => {
    var _a;
    if ((0, is_model_pivot_1.default)(model)) {
        return false;
    }
    if (relationField.isList || relatedField.isList) {
        return false;
    }
    if (definedAndNotEmpty(relationField.relationFromFields) ||
        definedAndNotEmpty(relationField.relationToFields) ||
        emptyOrUndefined(relatedField.relationFromFields) ||
        emptyOrUndefined(relatedField.relationToFields)) {
        return false;
    }
    if (relatedField.relationFromFields.length > 1 ||
        relatedField.relationToFields.length > 1) {
        throw new composite_key_on_relation_error_1.default();
    }
    const imports = new Set();
    const primaryKeyField = lodash_1.default.find(model.fields, f => f.isId);
    imports.add((0, get_model_fqcn_1.default)(relatedModel).fqcn);
    let relation = {
        name: relationField.name,
        className: (0, get_model_classname_1.default)(relatedModel),
    };
    if (explicitTableNames ||
        relatedField.relationFromFields[0] !==
            `${(0, snake_1.default)(model.name)}_${relatedField.relationToFields[0]}`) {
        relation = {
            ...relation,
            foreignKey: relatedField.relationFromFields[0],
        };
    }
    if (explicitTableNames ||
        relatedField.relationToFields[0] !== ((_a = primaryKeyField === null || primaryKeyField === void 0 ? void 0 : primaryKeyField.name) !== null && _a !== void 0 ? _a : 'id')) {
        relation = {
            ...relation,
            localKey: relatedField.relationToFields[0],
        };
    }
    return [relation, imports];
};
exports.getHasOneRelation = getHasOneRelation;
const getBelongsToRelation = (model, relationField, relatedModel, relatedField, explicitTableNames = true) => {
    var _a;
    if ((0, is_model_pivot_1.default)(model)) {
        return false;
    }
    if (relationField.isList) {
        return false;
    }
    if (emptyOrUndefined(relationField.relationFromFields) ||
        emptyOrUndefined(relationField.relationToFields) ||
        definedAndNotEmpty(relatedField.relationFromFields) ||
        definedAndNotEmpty(relatedField.relationToFields)) {
        return false;
    }
    if (relationField.relationFromFields.length > 1 ||
        relationField.relationToFields.length > 1) {
        throw new composite_key_on_relation_error_1.default();
    }
    const imports = new Set();
    const relatedPrimaryKeyField = lodash_1.default.find(relatedModel.fields, f => f.isId);
    imports.add((0, get_model_fqcn_1.default)(relatedModel).fqcn);
    let relation = {
        name: relationField.name,
        className: (0, get_model_classname_1.default)(relatedModel),
        nullable: !relationField.isRequired,
    };
    if (explicitTableNames ||
        relationField.relationFromFields[0] !==
            `${(0, snake_1.default)(relationField.name)}_${relationField.relationToFields[0]}`) {
        relation = {
            ...relation,
            foreignKey: relationField.relationFromFields[0],
        };
    }
    if (explicitTableNames ||
        relationField.relationToFields[0] !== ((_a = relatedPrimaryKeyField === null || relatedPrimaryKeyField === void 0 ? void 0 : relatedPrimaryKeyField.name) !== null && _a !== void 0 ? _a : 'id')) {
        relation = {
            ...relation,
            ownerKey: relationField.relationToFields[0],
        };
    }
    return [relation, imports];
};
exports.getBelongsToRelation = getBelongsToRelation;
const getHasManyRelation = (model, relationField, relatedModel, relatedField, explicitTableNames = true) => {
    var _a;
    if ((0, is_model_pivot_1.default)(model)) {
        return false;
    }
    if (!relationField.isList || relatedField.isList) {
        return false;
    }
    if (definedAndNotEmpty(relationField.relationFromFields) ||
        definedAndNotEmpty(relationField.relationToFields) ||
        emptyOrUndefined(relatedField.relationFromFields) ||
        emptyOrUndefined(relatedField.relationToFields)) {
        return false;
    }
    if (relatedField.relationFromFields.length > 1 ||
        relatedField.relationToFields.length > 1) {
        throw new composite_key_on_relation_error_1.default();
    }
    const imports = new Set();
    const primaryKeyField = lodash_1.default.find(model.fields, f => f.isId);
    imports.add((0, get_model_fqcn_1.default)(relatedModel).fqcn);
    let relation = {
        name: relationField.name,
        className: (0, get_model_classname_1.default)(relatedModel),
    };
    if (explicitTableNames ||
        relatedField.relationFromFields[0] !==
            `${(0, snake_1.default)(model.name)}_${relatedField.relationToFields[0]}`) {
        relation = {
            ...relation,
            foreignKey: relatedField.relationFromFields[0],
        };
    }
    if (explicitTableNames ||
        relatedField.relationToFields[0] !== ((_a = primaryKeyField === null || primaryKeyField === void 0 ? void 0 : primaryKeyField.name) !== null && _a !== void 0 ? _a : 'id')) {
        relation = {
            ...relation,
            localKey: relatedField.relationToFields[0],
        };
    }
    return [relation, imports];
};
exports.getHasManyRelation = getHasManyRelation;
const getBelongsToManyRelation = (model, relationField, relatedModel, relatedField, models, explicitTableNames = true) => {
    var _a, _b;
    const relations = [];
    if ((0, is_model_pivot_1.default)(model)) {
        return false;
    }
    if (!relationField.isList) {
        return false;
    }
    const isImplicit = relatedField.isList;
    if (!isImplicit && !(0, is_model_pivot_1.default)(relatedModel)) {
        return false;
    }
    const imports = new Set();
    const primaryKeyField = lodash_1.default.find(model.fields, f => f.isId);
    if (isImplicit) {
        if (definedAndNotEmpty(relationField.relationFromFields) ||
            definedAndNotEmpty(relatedField.relationFromFields)) {
            return false;
        }
        imports.add((0, get_model_fqcn_1.default)(relatedModel).fqcn);
        const relation = {
            name: relationField.name,
            className: (0, get_model_classname_1.default)(relatedModel),
            foreignPivotKey: model.name.localeCompare(relatedModel.name) < 0 ? 'A' : 'B',
            relatedPivotKey: model.name.localeCompare(relatedModel.name) > 0 ? 'A' : 'B',
            pivotTable: `_${relationField.relationName}`,
        };
        relations.push(relation);
    }
    else {
        if (definedAndNotEmpty(relationField.relationFromFields) ||
            definedAndNotEmpty(relationField.relationToFields) ||
            emptyOrUndefined(relatedField.relationFromFields) ||
            emptyOrUndefined(relatedField.relationToFields)) {
            return false;
        }
        if (relatedField.relationFromFields.length > 1 ||
            relatedField.relationToFields.length > 1) {
            throw new composite_key_on_relation_error_1.default();
        }
        const pivotRelationFields = lodash_1.default.filter(relatedModel.fields, f => (0, is_field_relation_1.default)(f) &&
            f.relationName !== relationField.relationName &&
            f.type !== relatedModel.name);
        for (const pivotRelationField of pivotRelationFields) {
            const relatedPivotModel = (0, get_model_by_name_1.default)(pivotRelationField.type, models);
            const relatedPivotField = (0, get_related_field_of_related_model_1.default)(relatedModel, relatedPivotModel, pivotRelationField);
            if (!relatedPivotField.isList) {
                continue;
            }
            if (emptyOrUndefined(pivotRelationField.relationFromFields) ||
                emptyOrUndefined(pivotRelationField.relationToFields) ||
                definedAndNotEmpty(relatedPivotField.relationFromFields) ||
                definedAndNotEmpty(relatedPivotField.relationToFields)) {
                continue;
            }
            const relatedPivotModelPrimaryKeyField = lodash_1.default.find(relatedPivotModel.fields, f => f.isId);
            imports.add((0, get_model_fqcn_1.default)(relatedPivotModel).fqcn);
            imports.add((0, get_model_fqcn_1.default)(relatedModel).fqcn);
            let relation = {
                name: relationField.name,
                className: (0, get_model_classname_1.default)(relatedPivotModel),
                using: relatedModel.name,
            };
            let snakeFrom = (0, snake_1.default)(model.name);
            let snakeTo = (0, snake_1.default)(relatedPivotModel.name);
            if (snakeFrom.localeCompare(snakeTo) > 0) {
                [snakeFrom, snakeTo] = [snakeTo, snakeFrom];
            }
            const pivotTable = (0, get_table_name_from_model_1.default)(relatedModel);
            if (explicitTableNames || pivotTable !== `${snakeFrom}_${snakeTo}`) {
                relation = {
                    ...relation,
                    pivotTable,
                };
            }
            if (explicitTableNames ||
                relatedField.relationFromFields[0] !==
                    `${(0, snake_1.default)(model.name)}_${relatedField.relationToFields[0]}`) {
                relation = {
                    ...relation,
                    foreignPivotKey: relatedField.relationFromFields[0],
                };
            }
            if (explicitTableNames ||
                pivotRelationField.relationFromFields[0] !==
                    `${(0, snake_1.default)(relatedPivotModel.name)}_${pivotRelationField.relationToFields[0]}`) {
                relation = {
                    ...relation,
                    relatedPivotKey: pivotRelationField.relationFromFields[0],
                };
            }
            if (explicitTableNames ||
                relatedField.relationToFields[0] !== ((_a = primaryKeyField === null || primaryKeyField === void 0 ? void 0 : primaryKeyField.name) !== null && _a !== void 0 ? _a : 'id')) {
                relation = {
                    ...relation,
                    parentKey: relatedField.relationToFields[0],
                };
            }
            if (explicitTableNames ||
                pivotRelationField.relationToFields[0] !==
                    ((_b = relatedPivotModelPrimaryKeyField === null || relatedPivotModelPrimaryKeyField === void 0 ? void 0 : relatedPivotModelPrimaryKeyField.name) !== null && _b !== void 0 ? _b : 'id')) {
                relation = {
                    ...relation,
                    relatedKey: pivotRelationField.relationToFields[0],
                };
            }
            relations.push(relation);
        }
    }
    return [relations, imports];
};
exports.getBelongsToManyRelation = getBelongsToManyRelation;
const getRelationsFromModel = (model, models, explicitTableNames = true) => {
    const imports = new Set();
    const hasOne = [];
    const belongsTo = [];
    const hasMany = [];
    const belongsToMany = [];
    const relationFields = lodash_1.default.filter(model.fields, f => (0, is_field_relation_1.default)(f));
    for (const relationField of relationFields) {
        const relatedModel = (0, get_model_by_name_1.default)(relationField.type, models);
        const relatedField = (0, get_related_field_of_related_model_1.default)(model, relatedModel, relationField);
        const hasOneRelation = (0, exports.getHasOneRelation)(model, relationField, relatedModel, relatedField, explicitTableNames);
        if (hasOneRelation) {
            const [relation, newImports] = hasOneRelation;
            hasOne.push(relation);
            for (const newImport of newImports) {
                imports.add(newImport);
            }
            continue;
        }
        const belongsToRelation = (0, exports.getBelongsToRelation)(model, relationField, relatedModel, relatedField, explicitTableNames);
        if (belongsToRelation) {
            const [relation, newImports] = belongsToRelation;
            belongsTo.push(relation);
            for (const newImport of newImports) {
                imports.add(newImport);
            }
            continue;
        }
        const belongsToManyRelation = (0, exports.getBelongsToManyRelation)(model, relationField, relatedModel, relatedField, models, explicitTableNames);
        if (belongsToManyRelation) {
            const [relations, newImports] = belongsToManyRelation;
            for (const relation of relations) {
                belongsToMany.push(relation);
            }
            for (const newImport of newImports) {
                imports.add(newImport);
            }
            continue;
        }
        const hasManyRelation = (0, exports.getHasManyRelation)(model, relationField, relatedModel, relatedField, explicitTableNames);
        if (hasManyRelation) {
            const [relation, newImports] = hasManyRelation;
            hasMany.push(relation);
            for (const newImport of newImports) {
                imports.add(newImport);
            }
            continue;
        }
    }
    return {
        imports,
        hasOne,
        belongsTo,
        hasMany,
        belongsToMany,
    };
};
exports.default = getRelationsFromModel;
//# sourceMappingURL=get-relations-from-model.js.map