"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.format = exports.formatFile = exports.checkPhpCsFixer = void 0;
const node_child_process_1 = require("node:child_process");
const node_path_1 = require("node:path");
const node_fs_1 = require("node:fs");
const promises_1 = require("node:fs/promises");
const node_util_1 = require("node:util");
const create_temporary_file_1 = __importDefault(require("../helpers/create-temporary-file"));
const exec = (0, node_util_1.promisify)(node_child_process_1.exec);
const checkPhpCsFixer = (phpCsFixerBin, phpCsFixerConfig) => {
    if (!(0, node_fs_1.existsSync)(phpCsFixerBin)) {
        throw new Error(`The provided path for php-cs-fixer does not exists: ${phpCsFixerBin}`);
    }
    if (!(0, node_fs_1.existsSync)(phpCsFixerConfig)) {
        throw new Error(`The provided path for php-cs-fixer's config does not exists: ${phpCsFixerConfig}`);
    }
    return true;
};
exports.checkPhpCsFixer = checkPhpCsFixer;
const formatFile = async (path, phpCsFixerBin = '../vendor/bin/php-cs-fixer', phpCsFixerConfig = '../.php-cs.dist.php') => {
    if (!(0, node_fs_1.existsSync)(path)) {
        throw new Error(`The requested file to be prettified does not exists: ${path}`);
    }
    (0, exports.checkPhpCsFixer)(phpCsFixerBin, phpCsFixerConfig);
    await exec(`"${phpCsFixerBin}" fix "${path}" --quiet --config="${phpCsFixerConfig}"`);
};
exports.formatFile = formatFile;
const format = async (content, phpCsFixerBin = '../vendor/bin/php-cs-fixer', phpCsFixerConfig = '../.php-cs.dist.php') => {
    (0, exports.checkPhpCsFixer)(phpCsFixerBin, phpCsFixerConfig);
    const filePath = await (0, create_temporary_file_1.default)(content);
    await (0, exports.formatFile)(filePath, phpCsFixerBin, phpCsFixerConfig);
    const prettifiedContent = await (0, promises_1.readFile)(filePath, { encoding: 'utf8' });
    await (0, promises_1.rm)((0, node_path_1.dirname)(filePath), { recursive: true, force: true });
    return prettifiedContent;
};
exports.format = format;
exports.default = { formatFile: exports.formatFile, format: exports.format };
//# sourceMappingURL=php-cs-fixer.js.map