"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const get_field_lines_1 = __importDefault(require("./get-field-lines"));
const get_default_field_attribute_1 = __importDefault(require("./get-default-field-attribute"));
const unsupportedMatcher = /^Unsupported\("(\\"|[^"])+"\)((\?)|(\[]))?$/g;
const uniqueMatcher = /@unique(\([^)]*\))?/g;
const idMatcher = /@id(\([^)]*\))?/g;
const updatedAtMatcher = /@updatedAt(\([^)]*\))?/g;
const defaultIsFunctionMatcher = /([^()]+)(\((("(\\"|[^"])+")|[^)])*\))/g;
const argumentsMatcher = /(("(\\"|[^"])*")|[^",]+)\s*(,\s*|$)/g;
const convertDefault = (defaultValue) => {
    const sanitizedDefaultValue = defaultValue.trim();
    const defaultIsFunction = defaultIsFunctionMatcher.exec(sanitizedDefaultValue);
    if (!defaultIsFunction) {
        return JSON.parse(sanitizedDefaultValue);
    }
    const result = {
        name: defaultIsFunction[1],
        args: [],
    };
    argumentsMatcher.lastIndex = 0;
    let match = argumentsMatcher.exec(defaultIsFunction[3]);
    while (match) {
        result.args.push(convertDefault(match[1]));
        match = argumentsMatcher.exec(defaultIsFunction[3]);
    }
    return result;
};
const getUnsupportedFields = (modelName, rawContent) => {
    var _a, _b, _c;
    const fieldLines = (0, get_field_lines_1.default)(modelName, rawContent);
    if (!fieldLines) {
        return false;
    }
    const fields = [];
    for (const fieldLine of fieldLines) {
        unsupportedMatcher.lastIndex = 0;
        const match = unsupportedMatcher.exec(fieldLine.type);
        if (match) {
            const defaultValue = (0, get_default_field_attribute_1.default)(fieldLine.id, modelName, rawContent);
            const field = {
                kind: 'unsupported',
                name: fieldLine.id,
                isRequired: !match[3],
                isList: !!match[4],
                isUnique: !!((_a = fieldLine.attributes) === null || _a === void 0 ? void 0 : _a.match(uniqueMatcher)),
                isId: !!((_b = fieldLine.attributes) === null || _b === void 0 ? void 0 : _b.match(idMatcher)),
                isReadOnly: false,
                isGenerated: false,
                isUpdatedAt: !!((_c = fieldLine.attributes) === null || _c === void 0 ? void 0 : _c.match(updatedAtMatcher)),
                type: fieldLine.type.replace(/\[]$/, '').replace(/\?$/, ''),
                hasDefaultValue: !!defaultValue,
                default: defaultValue ? convertDefault(defaultValue) : undefined,
                documentation: fieldLine.comment,
            };
            fields.push(field);
        }
    }
    return fields;
};
exports.default = getUnsupportedFields;
//# sourceMappingURL=get-unsupported-fields.js.map